/***********************************************************************
*                                                                      *
*    DataPlotWndProc.c - Data Plot Window                              *
*                                                                      *
************************************************************************
************************************************************************
*                                                                      *
*    Programmautor                                                     *
*      Dr. Rainer Sieger                                               *
*      Alfred-Wegener-Institut		                                     *
*                                                                      *
************************************************************************
*                                                                      *
*    letztes Editierdatum                                              *
*      22.05.2002                                                      *
*                                                                      *
***********************************************************************/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "Struct.h"
#include "Globals.h"

#include "Win_GUI.h"
#include "Dialog.h"
#include "WndProc.h"
#include "Do_FileMenu.h"
#include "Do_Batch.h"
#include "Do_YAxisMenu.h"
#include "Do_SetAllMenu.h"

#include "EnableMenu.h"
#include "DisableMenu.h"
#include "SetMenuItems.h"
#include "Ezfont.h"

#include "Window.h"
#include "Fileselect.h"
#include "ReadData.h"

#include "Preferences.h"
#include "Plotter.h"

#include "Do_GraphicDialog.h"
#include "Do_XAxisDialog.h"
#include "Do_YAxisDialog.h"

#include "PrintPicture.h"
#include "PrintTools.h"

#include "SetMinMax.h"
#include "ResetFilename.h"
#include "ExportPicture.h"
#include "SetMouseCursor.h"
#include "SetMemory.h"
#include "ShowStatusWindowMessages.h"

LRESULT CALLBACK DataPlotWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	static HWND   hStatusWnd;
	static HWND		hwndClient, hwndFrame;

	static int    nPaint;

	int           j        = 0;
	int           Mx       = 0;
	int           My       = 0;
	int           err      = 0;
	int           MenuItem = 0;

	HDC						hdc;
	PAINTSTRUCT		ps;
						
//------------------------------------------------------------------

	switch (iMsg)
	{
		case WM_CREATE :
			hwndClient			 = GetParent(hwnd);
			hwndFrame				 = GetParent(hwndClient);

			nPaint           = 0;

	 		InitCommonControls();

  		hStatusWnd      = CreateStatusWindow(WS_CHILD | WS_VISIBLE | CCS_BOTTOM | SBARS_SIZEGRIP, "", hwnd, 1);
			
			return 0;
			break;

		case WM_MDIACTIVATE :
			// Set the Data Plot menu if gaining focus
			if (lParam == (LPARAM) hwnd)
				SendMessage (hwndClient, WM_MDISETMENU, (WPARAM) hMenuDataPlot, (LPARAM) hMenuDataPlotWindow);

			// Set Item New Data Description Window 
			if (gDataDescriptionWindow != 0)
				EnableMenuItem(hMenuDataPlot, IDM_OPENDATADESCRIPTIONWINDOW, MF_GRAYED);		    // New Data Plot Window
			else
				EnableMenuItem(hMenuDataPlot, IDM_OPENDATADESCRIPTIONWINDOW, MF_ENABLED);		    // New Data Plot Window

			// Set the Init menu if losing focus
			if (lParam != (LPARAM) hwnd)
			{
				SendMessage (hwndClient, WM_MDISETMENU, (WPARAM) hMenuInit, (LPARAM) hMenuInitWindow);
				gActivWindow = DataPlotWindowIsNotActiv;                                 
			}

			DrawMenuBar (hwndFrame);

			return 0;
			break;

			case WM_QUERYENDSESSION :
			case WM_CLOSE :
				GetClientWindowPos(hwndFrame,hwnd,&gClientDataPlotWindowX,&gClientDataPlotWindowY,&gClientDataPlotWindowWidth,&gClientDataPlotWindowHeight);
				GetFrameWindowPos(hwndFrame,&gFrameWindowX,&gFrameWindowY,&gFrameWindowWidth,&gFrameWindowHeight);
				err = SaveWindowPreferences();

				if (gDataPlotWindow > 0)
				{
//		    err = Do_StopAlert(hwnd, rSaveGraphic);

					err = IDNO;

					switch (err)
					{
	    			case IDCANCEL: 
	    				return 0; 
	    				break;

						case IDYES: 
	    				err = Do_Saveas(hwnd, hInst, ExportPICT);
							gDataPlotWindow = 0;
	    				break;

						case IDNO:     
							gDataPlotWindow = 0;
							break;

						default:
							break;
					}
				}
				break;   // I.e., call DefMDIChildProc

			case WM_SIZE :
				GetWindowSize(hwnd,hStatusWnd,&gDataPlotWindowWidth,&gDataPlotWindowHeight);
				MoveWindow(hStatusWnd,0,0,LOWORD(lParam),HIWORD(lParam),TRUE);
				break;

			case WM_RBUTTONDOWN :
			case WM_LBUTTONDOWN :
				if (gActivWindow == DataPlotWindowIsNotActiv)
				{
					gActivWindow = DataPlotWindowIsActiv;
					return(0);
				}  
		    
				if ((gSelectedNumOfParameters > 0) || (Mode == TernaryMode))
				{
					Mx = LOWORD (lParam);
					My = HIWORD (lParam);
					Do_ContentClick (hwndFrame, hwnd, hStatusWnd, hInst, Mx, My); 
					EnableMenu (hwndFrame, hMenuDataPlot, hSubMenuFile, hSubMenuEdit, hSubMenuOptions, hSubMenuYAxis, hSubMenuSetAll);
					SetMenuItems (hSubMenuOptions);
				}
				return 0;
				break;

			case WM_PAINT :
				hdc = BeginPaint (hwnd, &ps);
				SetBkMode(hdc,TRANSPARENT);
 				GetLogicalResolution (hdc, TRUE, &cxDpi, &cyDpi);
				lf.lfHeight = -MulDiv(FontSize,GetDeviceCaps(hdc,LOGPIXELSY),72);

				ScreenDpi = (int) cxDpi;
				dpcm      = (int) (ScreenDpi/2.54f);

				Plotter(hdc,Init);

				EndPaint(hwnd, &ps);
				SetDefaultMouseCursor(); 
				return 0;
				break;

			case WM_MOUSEMOVE :
				Mx = LOWORD (lParam);
				My = HIWORD (lParam);
				ShowStatusWindowMessages(hStatusWnd, Mx, My);
				return 0;
				break;

			case WM_COMMAND :
				MenuItem = LOWORD (wParam);
				switch (MenuItem)
				{
	// Messages from File menu
					case IDM_SAVEAS :
						err = Do_Saveas (hwnd, hInst, SaveData);
						DoCaptionDataPlotWindow (hwnd, (char *) FullName);
						InvalidateRgn (hwnd, NULL, TRUE); 
						return 0;
						break;

					case IDM_EXPORT :
						err = Do_Saveas (hwnd, hInst, ExportPICT);
						DoCaptionDataPlotWindow (hwnd, (char *) FullName);
						InvalidateRgn (hwnd, NULL, TRUE); 
						return 0;
						break;

					case IDM_BATCH :
						err = Do_Batch (hwnd, hInst);
						return 0;
						break;

					case IDM_PAGESETUP :
  					psd.Flags = PSD_DISABLEMARGINS | PSD_INHUNDREDTHSOFMILLIMETERS;
						PageSetupDlg (&psd);
						return 0;
						break;

					case IDM_PRINT :
						PrintPicture (hwnd, 1);
						return 0;
						break;

					case IDM_PRINTNODIALOG :
						PrintPicture (hwnd, 0);
						return 0;
						break;

					case IDM_LOAD_TEMPLATE :
						err = Do_Open (hwnd, hInst, LoadTemplate);

						if (gNumOfParameters < 1)
							DisableMenu (hwndFrame, hMenuDataPlot, hSubMenuFile, hSubMenuEdit, hSubMenuOptions);
						else
						{
         			EnableMenu (hwndFrame, hMenuDataPlot, hSubMenuFile, hSubMenuEdit, hSubMenuOptions, hSubMenuYAxis, hSubMenuSetAll);
        			SetMenuItems (hSubMenuOptions);
						}

						DoCaptionDataPlotWindow (hwnd, (char *) FullName);
						InvalidateRgn (hwnd, NULL, TRUE); 
						return 0;
						break;

					case IDM_SAVE_TEMPLATE :
						err = Do_Saveas (hwnd, hInst, SaveTemplate);
						DoCaptionDataPlotWindow (hwnd, (char *) FullName);
						InvalidateRgn (hwnd, NULL, TRUE); 
						return 0;
						break;

					case IDM_SAVEPREFS :
						SavePreferences(1);
						return 0;
						break;

					case IDM_SETMEMORY:
						Do_SetMemory(hwnd, hInst);
						return 0;
						break;

// Messages from Edit menu
					case IDM_UNDO :
						return 0;
						break;

					case IDM_CUT :
						return 0;
						break;

					case IDM_COPY :
						err = ExportPicture (hwnd,TOCLIPBOARD);
						return 0;
						break;

					case IDM_PASTE :
						return 0;
						break;

					case IDM_CLEAR :
						return 0;
						break;

					case IDM_SELALL :
						return 0;
						break;

					case IDM_PLOTNEW :
						InvalidateRgn (hwnd, NULL, TRUE); 
						return 0;
						break;

	// Messages from Option menu
					case IDM_HorizontalMode :
						if (Mode == VerticalMode)
							ST[0].ReverseRange = abs(ST[0].ReverseRange-1); 

						Mode											= HorizontalMode;
						gSelectedNumOfParameters	= 0;

						for (j=1;j<=MAXNUMOFPARAMETERS;j++)
						{
							if (ST[j].PlotData > 0)
 								gSelectedNumOfParameters++;
						}

						EnableMenu (hwndFrame, hMenuDataPlot, hSubMenuFile, hSubMenuEdit, hSubMenuOptions, hSubMenuYAxis, hSubMenuSetAll);
      			SetMenuItems (hSubMenuOptions);
 						InvalidateRgn (hwnd, NULL, TRUE);
						return 0;
						break;

					case IDM_VerticalMode :
						if (Mode == HorizontalMode)
							ST[0].ReverseRange = abs(ST[0].ReverseRange-1); 

						Mode											= VerticalMode;
						gSelectedNumOfParameters	= 0;

						for (j=1;j<=MAXNUMOFPARAMETERS;j++)
						{
							if (ST[j].PlotData > 0)
 								gSelectedNumOfParameters++;
						}

						EnableMenu (hwndFrame, hMenuDataPlot, hSubMenuFile, hSubMenuEdit, hSubMenuOptions, hSubMenuYAxis, hSubMenuSetAll);
      			SetMenuItems (hSubMenuOptions);
 						InvalidateRgn (hwnd, NULL, TRUE);
						return 0;
						break;

					case IDM_TernaryMode :
						Mode 											= TernaryMode;
						SelectMode								= useGroups;
						gSelectedNumOfParameters	= 0;

						for (j=1;j<=MAXNUMOFPARAMETERS;j++)
						{
							if (ST[j].GPlotData > 0)
 								gSelectedNumOfParameters++;
						}

      			EnableMenu (hwndFrame, hMenuDataPlot, hSubMenuFile, hSubMenuEdit, hSubMenuOptions, hSubMenuYAxis, hSubMenuSetAll);
      			SetMenuItems (hSubMenuOptions);
 						InvalidateRgn (hwnd, NULL, TRUE);
						return 0;
						break;

					case IDM_PhaseMode :
						Mode 											= PhaseMode;
						SelectMode								= useGroups;
						gSelectedNumOfParameters	= 0;
						for (j=1;j<=MAXNUMOFPARAMETERS;j++)
						{
							if (ST[j].GPlotData > 0)
 								gSelectedNumOfParameters++;
						}

						EnableMenu (hwndFrame, hMenuDataPlot, hSubMenuFile, hSubMenuEdit, hSubMenuOptions, hSubMenuYAxis, hSubMenuSetAll);
      			SetMenuItems (hSubMenuOptions);
 						InvalidateRgn (hwnd, NULL, TRUE);
						return 0;
						break;

					case IDM_GRAPHIC :
						switch (Mode)
						{
							case VerticalMode:
							case HorizontalMode:
								if (DialogBox (hInst, "GRAPHICDIALOG", hwnd, (DLGPROC) Do_GraphicDialog))
								{
									GetClientWindowPos(hwndFrame,hwnd,&gClientDataPlotWindowX,&gClientDataPlotWindowY,&gClientDataPlotWindowWidth,&gClientDataPlotWindowHeight);
									ResizeWindow (hwndFrame, hwnd, hStatusWnd, gClientDataPlotWindowX, gClientDataPlotWindowY, gDataPlotWindowWidth, gDataPlotWindowHeight);
									InvalidateRgn (hwnd, NULL, TRUE); 
								}
								break;

							case TernaryMode:
							case PhaseMode:
								if (DialogBox (hInst, "TERGRAPHICDIALOG", hwnd, (DLGPROC) Do_GraphicDialog))
								{
									GetClientWindowPos(hwndFrame,hwnd,&gClientDataPlotWindowX,&gClientDataPlotWindowY,&gClientDataPlotWindowWidth,&gClientDataPlotWindowHeight);
									ResizeWindow (hwndFrame, hwnd, hStatusWnd, gClientDataPlotWindowX, gClientDataPlotWindowY, gDataPlotWindowWidth, gDataPlotWindowHeight);
									InvalidateRgn (hwnd, NULL, TRUE); 
								}
								break;

								default:
									break;
						}

						EnableMenu (hwndFrame, hMenuDataPlot, hSubMenuFile, hSubMenuEdit, hSubMenuOptions, hSubMenuYAxis, hSubMenuSetAll);
      			SetMenuItems (hSubMenuOptions);
						return 0;
						break;

					case IDM_XAXIS :
						DialogBox (hInst, "XAXISDIALOG", hwnd, (DLGPROC) Do_XAxisDialog);
            SetMinMax (-1);  	
						InvalidateRgn (hwnd, NULL, TRUE); 
						return 0;
						break;

						case IDM_YAXIS_SELECTBYPARAMETERS :
						case IDM_YAXIS_SELECTBYGROUPS :
						case IDM_YAXIS_SELALL :
						case IDM_YAXIS_SELNONE :
							Do_YAxisMenu (hwnd, hInst, MenuItem);
      				SetMenuItems (hSubMenuOptions);
							InvalidateRgn (hwnd, NULL, TRUE); 
							return 0;
							break;

						case IDM_YAXIS_SETALL :
						case IDM_YAXIS_MAXAXISLENGTH :
						case IDM_YAXIS_ADDNEXT : 
						case IDM_YAXIS_PARAMETER_TEXT_ORIENTATION : 
						case IDM_YAXIS_RANGE : 
						case IDM_YAXIS_TICKS : 
						case IDM_YAXIS_MARKER : 
						case IDM_YAXIS_OPTIONS : 
						case IDM_YAXIS_QUALITY_FLAGS : 
						case IDM_YAXIS_COLOR : 
						case IDM_YAXIS_COLOR_MAPPING : 
						case IDM_YAXIS_LINEWIDTH : 
						case IDM_YAXIS_COLUMNWIDTH : 
						case IDM_GEPSILON : 
							Do_SetAllMenu (hwnd, hInst, MenuItem);
      				SetMenuItems (hSubMenuOptions);
							InvalidateRgn (hwnd, NULL, TRUE); 
							return 0;
							break;

						default:
							break;
					}

			case WM_DESTROY :
				GlobalFree (pd.hDevMode);
				GlobalFree (pd.hDevNames);
				GlobalFree (psd.hDevMode);
				GlobalFree (psd.hDevNames);
				return 0;
				break;

			default:
				break;
	}

	return(DefMDIChildProc (hwnd, iMsg, wParam, lParam));
}

/*********************************************************************/
/*********************************************************************/
/*********************************************************************/

void DoCaptionDataPlotWindow (HWND hwnd, char *szTitleName)
{
	char szCaption[64 + 256];
	wsprintf (szCaption, "Data plot - %s", szTitleName[0] ? szTitleName : UNTITLED);
	SetWindowText (hwnd, szCaption);
}


/*********************************************************************/
/*********************************************************************/
/*********************************************************************/

HWND CreateDataPlotWindow(HWND hwndClient, HWND *hwndChild)
{	
	MDICREATESTRUCT     mdicreate;
	RECT	              ClientRect;

	mdicreate.szClass = szDataPlotClass;
	mdicreate.szTitle = "Data Plot";
	mdicreate.hOwner  = hInst;

	if ((gClientDataPlotWindowWidth == -1) && (gClientDataPlotWindowHeight == -1))
	{
		GetClientRect(hwndClient,&ClientRect);

		if (gNumOfLines > 0)
		{
 			gClientDataPlotWindowWidth  = ClientRect.right - ClientRect.left - 30;
			gClientDataPlotWindowHeight = ClientRect.bottom - ClientRect.top - 28;
			mdicreate.x    = gClientDataPlotWindowX = 25;
			mdicreate.y    = gClientDataPlotWindowY = 28;
			mdicreate.cx   = gClientDataPlotWindowWidth;
			mdicreate.cy   = gClientDataPlotWindowHeight;
		}
		else
		{
 			gClientDataPlotWindowWidth  = ClientRect.right - ClientRect.left - 10;
			gClientDataPlotWindowHeight = ClientRect.bottom - ClientRect.top - 10;
			mdicreate.x    = gClientDataPlotWindowY = 5;
			mdicreate.y    = gClientDataPlotWindowY = 5;
			mdicreate.cx   = gClientDataPlotWindowWidth;
			mdicreate.cy   = gClientDataPlotWindowHeight;
		}
	}
	else
	{
		mdicreate.x    = gClientDataPlotWindowX;
		mdicreate.y    = gClientDataPlotWindowY;
		mdicreate.cx   = gClientDataPlotWindowWidth;
		mdicreate.cy   = gClientDataPlotWindowHeight;
	}

	mdicreate.style  = 0;
	mdicreate.lParam = 0;

	*hwndChild = (HWND) SendMessage (hwndClient, WM_MDICREATE, 0, (LPARAM) (LPMDICREATESTRUCT) &mdicreate);

	gDataPlotWindow = 1;

	return(0);
}
