/*--------------------------------------------*/
/*  Do_XAxisDialog - Dialog fuer X-Achse      */
/*  Dr. Rainer Sieger, Universitt Bremen     */
/*  14.04.99                                  */
/*--------------------------------------------*/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "Struct.h"
#include "Globals.h"

#include "Win_GUI.h"
#include "Dialog.h"
#include "Do_XAxisDialog.h"

#include "Color.h"
#include "SetMinMax.h"
#include "Plotter.h"
#include "Julday.h"

#include "replaceStr.h"
#include "CreateFormat.h"

BOOL CALLBACK Do_XAxisDialog (HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	static float	    Xmin;
	static float	    Xmax;
	static float	    XRefline;
	static float      TickInkrement;    

	static int        MinorMarks;

	static int        dXAuto;
	static int				dXReverseRange;
	static int        dBothSide;
	static int        dInside;
	static int        dTicklines;
	static int        dMTicklines;
	static int        dClipAxis;
	static int        dFixedParameterWidth;

	static int				dim;

	static	float     resParameterWidth;    
	static	float     res;

	static Str255     dLabel					= "";

	char							DateTimeStr[32] = "";
	char							iFormatStr[32]  = "%d";
	char							fFormatStr[32]  = "%.1f";
	char							cFormatStr[32]  = "";

	BOOL							err							= TRUE;

	float							Time						= (float) 0.;

	int								Year						= 0;
	int								Month						= 0;
	int								Day							= 0;      

	int								Hour						= 0;
	int								Minute					= 0;      
	float 						Second					= 0;

	static HWND				hMinorMarks 	  = NULL;
	static HWND				hEditMinorMarks = NULL;
		
// *************************************************************************

	switch (iMsg)
  {
		case WM_INITDIALOG :
			dim = 0;
			strcpy(cFormatStr, (char *) ST[0].FormatStr);
			replaceStr(cFormatStr, "%.","");
			replaceStr(cFormatStr, "f","");
			sscanf(cFormatStr,"%d",&dim);
			dim++;
			sprintf(cFormatStr,"%%.%df", dim);

			res               = (float) cxDpi/2.54f;
			resParameterWidth = (float) ST[0].ParameterWidth/res;

			dXAuto							 = ST[0].SetDefaultRange;
			dXReverseRange			 = ST[0].ReverseRange;
			dBothSide						 = ST[0].BothSide;
			dInside							 = ST[0].Inside;
			dTicklines					 = ST[0].Tickline;
			dMTicklines					 = ST[0].Minor_Tickline;
			dFixedParameterWidth = ST[0].fixedParameterWidth;
			dClipAxis						 = ClipCoorSys;

			SetCheckButton(hDlg, IDC_XAUTO, Set, &dXAuto);
			SetCheckButton(hDlg, IDC_XREVERSE, Set, &dXReverseRange);
			SetCheckButton(hDlg, IDC_XBOTHSIDE, Set, &dBothSide);
			SetCheckButton(hDlg, IDC_XINSIDE, Set, &dInside);
			SetCheckButton(hDlg, IDC_XTICKLINES, Set, &dTicklines);
			SetCheckButton(hDlg, IDC_MXTICKLINES, Set, &dMTicklines);
			SetCheckButton(hDlg, IDC_CLIPAXIS, Set, &dClipAxis);
			SetCheckButton(hDlg, IDC_FIXEDPARAMETER, Set, &dFixedParameterWidth);
	    
			TextDialog(hDlg, IDC_LABEL, Set, (char *) C[0].Header);

			if ((!isDate) && (!isTime))
			{
				Xmin			= FloatDialog(hDlg, IDC_XMIN, Set, cFormatStr, ST[0].min, err);
				Xmax			= FloatDialog(hDlg, IDC_XMAX, Set, cFormatStr, ST[0].max, err);
				XRefline	= FloatDialog(hDlg, IDC_XREFLINE, Set, cFormatStr, ST[0].Refline, err);
			}
			else
			{
				CreateDateTimeStr(ST[0].min,(char *) DateTimeStr, 1);
				TextDialog(hDlg, IDC_XMIN, Set, (char *) DateTimeStr);
				CreateDateTimeStr(ST[0].max,(char *) DateTimeStr, 1);
				TextDialog(hDlg, IDC_XMAX, Set, (char *) DateTimeStr);
				CreateDateTimeStr(ST[0].Refline,(char *) DateTimeStr, 1);
				TextDialog(hDlg, IDC_XREFLINE, Set, (char *) DateTimeStr);
			}

			TickInkrement			= FloatDialog(hDlg, IDC_XTICKINCREMENT, Set, cFormatStr, ST[0].Tick_Increment, err);
			resParameterWidth = FloatDialog(hDlg, IDC_PARAMETERWIDTH, Set, fFormatStr, resParameterWidth, err);
			MinorMarks				= IntegerDialog(hDlg, IDC_XMINORMARKS, Set, iFormatStr, ST[0].Minor_Ticks, err);
			InitUpDownControl(hDlg,IDC_XMINORMARKS,1,100,MinorMarks,&hMinorMarks,&hEditMinorMarks);

			SetFocus(hDlg);
			return FALSE;
			break;

		case WM_VSCROLL :
			if ((HWND) lParam == hMinorMarks) UpDown(hMinorMarks,wParam);
			return FALSE;
	      
		case WM_COMMAND :
			switch (LOWORD (wParam))
			{
				case IDOK :
					ST[0].SetDefaultRange			= dXAuto;
					ST[0].ReverseRange				= dXReverseRange;
					ST[0].BothSide						= dBothSide;
					ST[0].Inside							= dInside;
					ST[0].Tickline						= dTicklines;
					ST[0].Minor_Tickline			= dMTicklines;
					ST[0].fixedParameterWidth = dFixedParameterWidth;
					ClipCoorSys								= dClipAxis;

					TextDialog(hDlg, IDC_LABEL, Change, (char *) C[0].Header);

					if ((!isDate) && (!isTime))
					{
						ST[0].min     = FloatDialog(hDlg, IDC_XMIN, Change, cFormatStr, Xmin, err);
						ST[0].max     = FloatDialog(hDlg, IDC_XMAX, Change, cFormatStr, Xmax, err);
						ST[0].Refline = FloatDialog(hDlg, IDC_XREFLINE, Change, cFormatStr, XRefline, err);
					}
					else
					{
						Time = (float) 0.;
						TextDialog(hDlg, IDC_XMIN, Change, (char *) DateTimeStr);

						if (StrToDate((char *) DateTimeStr,&Year,&Month,&Day) > 0)
				  		ST[0].min = (float) CalcJulDay(Year,Month,Day) - DateOffset;

						if (StrToTime((char *) DateTimeStr,&Hour,&Minute,&Second) > 0)
  						ST[0].min += CalcJulTime(Hour,Minute,Second);

						Time = (float) 0.;
						TextDialog(hDlg, IDC_XMAX, Change, (char *) DateTimeStr);
						
						if (StrToDate((char *) DateTimeStr,&Year,&Month,&Day) > 0)
				  		ST[0].max = (float) CalcJulDay(Year,Month,Day) - DateOffset;
						
						if (StrToTime((char *) DateTimeStr,&Hour,&Minute,&Second) > 0)
  						ST[0].max += CalcJulTime(Hour,Minute,Second);

						Time = (float) 0.;
						TextDialog(hDlg, IDC_XREFLINE, Change, (char *) DateTimeStr);
						
						if (StrToDate(DateTimeStr,&Year,&Month,&Day) > 0)
				  		ST[0].Refline = (float) CalcJulDay(Year,Month,Day) - DateOffset;
						
						if (StrToTime(DateTimeStr,&Hour,&Minute,&Second) > 0)
  						ST[0].Refline += CalcJulTime(Hour,Minute,Second);
					}

					ST[0].Tick_Increment = FloatDialog(hDlg, IDC_XTICKINCREMENT, Change, cFormatStr, TickInkrement, err);
					ST[0].Minor_Ticks    = IntegerDialog(hDlg, IDC_XMINORMARKS, Change, iFormatStr, MinorMarks, err);

					resParameterWidth    = FloatDialog(hDlg, IDC_PARAMETERWIDTH, Change, fFormatStr, resParameterWidth, err);
					ST[0].ParameterWidth = (int) (resParameterWidth*res);
					
					if (ST[0].ParameterWidth < 0)
					{
						ST[0].ParameterWidth			 = 0;
						ST[0].fixedParameterWidth = 0;
					}

					strcpy((char *) ST[0].FormatStr, cFormatStr);
					EndDialog (hDlg, TRUE);
					SetMinMax (-1);  
					return TRUE;
					break;

				case IDC_PARAMETERWIDTH :
					if (!(err = CheckValue(hDlg, IDC_PARAMETERWIDTH)))
						resParameterWidth = FloatDialog(hDlg, IDC_PARAMETERWIDTH, Set, fFormatStr, resParameterWidth, err);
					else
						resParameterWidth = FloatDialog(hDlg, IDC_PARAMETERWIDTH, Change, fFormatStr, resParameterWidth, err);
					return TRUE;
					break;

				case IDC_XMIN :
					if (!(err = CheckValue(hDlg, IDC_XMIN)))
						Xmin = FloatDialog(hDlg, IDC_XMIN, Set, cFormatStr, Xmin, err);
					else
						Xmin = FloatDialog(hDlg, IDC_XMIN, Change, cFormatStr, Xmin, err);
					return TRUE;
					break;

				case IDC_XMAX :
					if (!(err = CheckValue(hDlg, IDC_XMAX)))
						Xmax = FloatDialog(hDlg, IDC_XMAX, Set, cFormatStr, Xmax, err);
					else
						Xmax = FloatDialog(hDlg, IDC_XMAX , Change, cFormatStr, Xmax, err);
					return TRUE;
					break;

				case IDC_XREFLINE :
					if (!(err = CheckValue(hDlg, IDC_XREFLINE)))
						XRefline = FloatDialog(hDlg, IDC_XREFLINE, Set, cFormatStr, XRefline, err);
					else
						XRefline = FloatDialog(hDlg, IDC_XREFLINE , Change, cFormatStr, XRefline, err);
					return TRUE;
					break;

				case IDC_XTICKINCREMENT :
					if (!(err = CheckValue(hDlg, IDC_XTICKINCREMENT)))
						TickInkrement = FloatDialog(hDlg, IDC_XTICKINCREMENT, Set, cFormatStr, TickInkrement, err);
					else
						TickInkrement = FloatDialog(hDlg, IDC_XTICKINCREMENT , Change, cFormatStr, TickInkrement, err);
					return TRUE;
					break;

				case IDC_XMINORMARKS :
					if (!(err = CheckValue(hDlg, IDC_XMINORMARKS)))
						MinorMarks = IntegerDialog(hDlg, IDC_XMINORMARKS, Set, iFormatStr, MinorMarks, err);
					else
						MinorMarks = IntegerDialog(hDlg, IDC_XMINORMARKS , Change, iFormatStr, MinorMarks, err);
					return TRUE;
					break;

				case IDC_FIXEDPARAMETER :
					SetCheckButton(hDlg, IDC_FIXEDPARAMETER, Change, &dFixedParameterWidth);
					return TRUE;
					break;

				case IDC_XAUTO :
					SetCheckButton(hDlg, IDC_XAUTO, Change, &dXAuto);
					return TRUE;
					break;

				case IDC_XREVERSE :
					SetCheckButton(hDlg, IDC_XREVERSE, Change, &dXReverseRange);
					return TRUE;
					break;

				case IDC_XBOTHSIDE :
					SetCheckButton(hDlg, IDC_XBOTHSIDE, Change, &dBothSide);
					return TRUE;
					break;

				case IDC_XINSIDE :
					SetCheckButton(hDlg, IDC_XINSIDE, Change, &dInside);
					return TRUE;
					break;

				case IDC_XTICKLINES :
					SetCheckButton(hDlg, IDC_XTICKLINES, Change, &dTicklines);
					return TRUE;
					break;

				case IDC_MXTICKLINES :
					SetCheckButton(hDlg, IDC_MXTICKLINES, Change, &dMTicklines);
					return TRUE;
					break;

				case IDC_CLIPAXIS :
					SetCheckButton(hDlg, IDC_CLIPAXIS, Change, &dClipAxis);
					return TRUE;
					break;

				case IDCANCEL :
					EndDialog (hDlg, FALSE);
					return TRUE;
					break;

				default:
					break;
				}
			break;

		case WM_PAINT :
			break;

		default:
			break;
  }

	return FALSE;
}
