/*----------------------------------------------*/
/*  Dr. Rainer Sieger, Alfred-Wegener-Institut  */
/*  26.06.2001                                  */
/*----------------------------------------------*/


/*------------------------------------------
   FILESELECT.C -- File Functions
  ------------------------------------------*/

#include <windows.h>
#include <commdlg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <shlobj.h>

#include "Struct.h"
#include "Globals.h"

#include "Win_GUI.h"
#include "Fileselect.h"

static char szPPFilter[] = "PANGAEA Files (*.tab)\0*.tab\0" \
				  								 "Text Files (*.txt)\0*.txt\0" \
													 "Text Files (*.csv)\0*.csv\0" \
													 "ASCII Files (*.asc)\0*.asc\0" \
													 "All Files (*.*)\0*.*\0\0";

static char szTABFilter[] = "PANGAEA Files (*.tab)\0*.tab\0" \
				  								  "Text Files (*.txt)\0*.txt\0" \
													  "Text Files (*.csv)\0*.csv\0" \
													  "ASCII Files (*.asc)\0*.asc\0" \
													  "All Files (*.*)\0*.*\0\0";

static char szTXTFilter[] = "Text Files (*.txt)\0*.txt\0"  \
									  				"Text Files (*.csv)\0*.csv\0" \
													  "ASCII Files (*.asc)\0*.asc\0" \
													  "All Files (*.*)\0*.*\0\0";

static char szALLFilter[] = "All Files (*.*)\0*.*\0\0" \
														"PANGAEA Files (*.tab)\0*.tab\0" \
				  								  "Text Files (*.txt)\0*.txt\0" \
													  "Text Files (*.csv)\0*.csv\0" \
													  "ASCII Files (*.asc)\0*.asc\0";

static char szPTFFilter[] = "PanPlot Template Files (*.ptf)\0*.ptf\0"  \
  													"All Files (*.*)\0*.*\0\0";

static char szPPPFilter[] = "Microsoft Enhanced Metafiles (*.emf)\0*.emf\0"  \
  													"All Files (*.*)\0*.*\0\0";

static char szEXEFilter[] = "Program Files (*.EXE)\0*.EXE\0";

// ----------------------------------------------------------------------------

void FileInitialize (HWND hwnd, int iFileFilter)
{
	ofn.lStructSize       = sizeof (OPENFILENAME);
	ofn.hwndOwner         = hwnd;
	ofn.hInstance         = NULL;
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter    = 0;
	ofn.nFilterIndex      = 0;
	ofn.lpstrFile         = NULL;          // Set in Open and Close functions
	ofn.nMaxFile          = MAX_PATH + MAX_FNAME;
	ofn.lpstrFileTitle    = NULL;          // Set in Open and Close functions
	ofn.nMaxFileTitle     = MAX_FNAME;
	ofn.lpstrInitialDir   = NULL;
	ofn.lpstrTitle        = NULL;
	ofn.Flags             = 0;             // Set in Open and Close functions
	ofn.nFileOffset       = 0;
	ofn.nFileExtension    = 0;
	ofn.lpstrDefExt       = "pp";
	ofn.lCustData         = 0L;
	ofn.lpfnHook          = NULL;
	ofn.lpTemplateName    = NULL;

	switch (iFileFilter)
	{
		case TXT :
			ofn.lpstrFilter = szTXTFilter;
			break;

		case TAB :
			ofn.lpstrFilter = szTABFilter;
			break;

		case ALL :
			ofn.lpstrFilter = szALLFilter;
			break;

		case PTF :
			ofn.lpstrFilter = szPTFFilter;
			break;

		case PPP :
			ofn.lpstrFilter = szPPPFilter;
			break;

		case EXE :
			ofn.lpstrFilter = szEXEFilter;
			break;

		default:
			ofn.lpstrFilter = szTXTFilter;
			break;
	}
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

BOOL FileOpenDlg (HWND hwnd, HINSTANCE hInst, PSTR pstrFullName, PSTR pstrPath, PSTR pstrName, PSTR pstrTitle)									
{
	BOOL err = FALSE;

  HWND hChild;
	RECT rect;
  int  top;
  int  left;

// ----------------------------------------------------------------------------

  GetWindowRect (hwnd, &rect);
  top  = rect.top  + 30;
  left = rect.left +  5;
  GetClientRect (hwnd, &rect);

  hChild = CreateWindow ("static", "", WS_DISABLED, left,top, 420, 295, hwnd, NULL, hInst, NULL);

  ofn.hwndOwner         = hChild;
  ofn.lpstrFile         = pstrFullName;
  ofn.lpstrFileTitle    = pstrName;
  ofn.lpstrTitle        = pstrTitle;
  ofn.Flags             = OFN_HIDEREADONLY | OFN_CREATEPROMPT | OFN_LONGNAMES;

	if((err = GetOpenFileName (&ofn))) 
	{
		memset(pstrPath, (int) '\0', MAX_PATH + 1);
		strncpy(pstrPath, pstrFullName, ofn.nFileOffset);
	}

  DestroyWindow(hChild);
  SetFocus(hwnd);

  return(err);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

int MultiFileOpenDlg (HWND hwnd, HINSTANCE hInst)
{
	BOOL err = FALSE;

  HWND hChild;
	RECT rect;
  int  top;
  int  left;
	char *p;
	int  i = -1;

// ----------------------------------------------------------------------------

	GetWindowRect (hwnd, &rect);
  top  = rect.top;
  left = rect.left;
  GetClientRect (hwnd, &rect);
  top  += (rect.bottom - rect.top)/2 - 160;
  left += (rect.right - rect.left)/2 - 210;

  hChild = CreateWindow ("static", "", WS_DISABLED, left,top, 420, 295, hwnd, NULL, hInst, NULL);

	memset(InputStr, (int) '\0', MAXNUMOFCHAR+1);
  ofn.hwndOwner         = hChild;
  ofn.lpstrFile         = InputStr;
  ofn.lpstrFileTitle    = NULL;
  ofn.lpstrTitle        = "Select files";
  ofn.Flags             = OFN_EXPLORER | OFN_HIDEREADONLY | OFN_ALLOWMULTISELECT;

	if((err = GetOpenFileName (&ofn))) 
	{
		while ((p = strchr(InputStr, (int) '\0')) && (++i < MAXNUMOFCHAR))
		{
	    *p = (int) '|';
		
			if ((p = strstr(InputStr, "||")))
			{
		    *p = (int) '\0';
				i  = MAXNUMOFCHAR;
			}
		}
				
		i = -1;

		while ((p = strchr(InputStr, (int) '|')) && (++i < MAXNUMOFFILES))
    {
      memset((char *) Filelist[i], (int) '\0', MAX_FNAME);
      strncpy((char *) Filelist[i], (char *) InputStr, (unsigned int) (p-InputStr)); 
			strncpy(InputStr,p+1,strlen(p));
    } 

		if ((strlen(InputStr) > 0) && (++i < MAXNUMOFFILES))
			strcpy((char *) Filelist[i], (char *) InputStr);
	}

  DestroyWindow(hChild);
  SetFocus(hwnd);

  return(i);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

BOOL FileSaveDlg (HWND hwnd, HINSTANCE hInst,PSTR pstrFullName, PSTR pstrPath, PSTR pstrName, PSTR pstrTitle)
{
	BOOL err = FALSE;

  HWND hChild;
	RECT rect;
  int  top;
  int  left;

// ----------------------------------------------------------------------------

	GetWindowRect (hwnd, &rect);
  top  = rect.top;
  left = rect.left;
  GetClientRect (hwnd, &rect);
  top  += (rect.bottom - rect.top)/2 - 160;
  left += (rect.right - rect.left)/2 - 210;

  hChild = CreateWindow ("static", "", WS_DISABLED, left,top, 420, 295, hwnd, NULL, hInst, NULL);

  ofn.hwndOwner         = hChild;
  ofn.lpstrFile         = pstrFullName;
  ofn.lpstrFileTitle    = pstrName;
  ofn.lpstrTitle        = pstrTitle;
  ofn.Flags             = OFN_HIDEREADONLY | OFN_CREATEPROMPT | OFN_LONGNAMES | OFN_OVERWRITEPROMPT;

	if((err = GetSaveFileName (&ofn))) 
	{
		memset(pstrPath, (int) '\0', MAX_PATH + 1);
		strncpy(pstrPath, pstrFullName, ofn.nFileOffset);
	}

  DestroyWindow(hChild);
  SetFocus(hwnd);

  return(err);
}


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

int CALLBACK BrowseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData) 
{
	TCHAR szDir[MAX_PATH];

	switch(uMsg) 
	{
		case BFFM_INITIALIZED: 
			if (GetCurrentDirectory(sizeof(szDir)/sizeof(TCHAR),szDir)) 
			{
				// WParam is TRUE since you are passing a path.
				// It would be FALSE if you were passing a pidl.
				SendMessage(hwnd,BFFM_SETSELECTION,TRUE,(LPARAM)szDir);
			}
			break;

		case BFFM_SELCHANGED:
		// Set the status window to the currently selected path.
			if (SHGetPathFromIDList((LPITEMIDLIST) lp ,szDir))
				SendMessage(hwnd,BFFM_SETSTATUSTEXT,0,(LPARAM) szDir);
			break;

		default:
			break;
	}
	
	return(0);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// 26.2.99

int GetFolder(char *Path, StrName Filelist[MAXNUMOFFILES+1], int Mode, char *Title)
{
   BROWSEINFO bi;
   LPITEMIDLIST pidl;
   LPMALLOC pMalloc;
	 
	 WIN32_FIND_DATA  fd;
	 HANDLE						hFind;
	 StrFullName			lpFilename;
	 BOOL							bRet = TRUE;

	 int							i = 0;

// ----------------------------------------------------------------------------

   if (SUCCEEDED(SHGetMalloc(&pMalloc))) 
	 {
     ZeroMemory(&bi,sizeof(bi));
     bi.hwndOwner      = NULL;
     bi.pidlRoot       = 0;
     bi.pszDisplayName = 0;
		 bi.lpszTitle      = Title;
     bi.ulFlags        = BIF_RETURNONLYFSDIRS; // | BIF_STATUSTEXT;
     bi.lpfn           = BrowseCallbackProc;

     pidl = SHBrowseForFolder(&bi);

		 if (pidl) 
		 {
       if (!SHGetPathFromIDList(pidl,Path))
			 {
		     pMalloc->lpVtbl->Free(pMalloc,pidl);
	       pMalloc->lpVtbl->Release(pMalloc);
         return(0);
			 }
			 else
			 {
		     pMalloc->lpVtbl->Free(pMalloc,pidl);
	       pMalloc->lpVtbl->Release(pMalloc);

				 if (Mode > 0)
				 {
					strcpy((char *) lpFilename, Path);
					strcat((char *) lpFilename, "\\*.*");

					hFind = FindFirstFile((char *) lpFilename, &fd);

					while (hFind != INVALID_HANDLE_VALUE && bRet)
					{
						if ((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
						{
							if (++i < MAXNUMOFFILES)
							{
							 memset((char *) Filelist[i], (int) '\0', MAX_FNAME);
							 strcpy((char *) Filelist[i], fd.cFileName);
							}
						}

						bRet = FindNextFile(hFind, &fd);
					}

					FindClose(hFind);
				 }
			 }
     }
   }

	 return(i);
}
