/************************************************************************
*                                                                      *
*    Datei                                                             *
*      Graphic                                                         *
*                                                                      *
************************************************************************
************************************************************************
*                                                                      *
*    Programmautor                                                     *
*      Dr. Rainer Sieger                                               *
*      e-mail: rsieger@pangaea.de                                      *
*                                                                      *
************************************************************************
*                                                                      *
*    letztes Editierdatum                                              *
*      01.03.2001                                                      *
*                                                                      *
***********************************************************************/

#include <windows.h>
#include <math.h>

#include "Struct.h"
#include "Globals.h"

#include "Graphic.h"
#include "Win_GUI.h"

/**********************************************************************
***********************************************************************
**********************************************************************/

void CBoundsRechteckV(HDC hdc, int x1, int y1, int x2, int y2, int LineWidth, int LineType)
{  
	POINT  pt[2];

	if (LineType > 0)
	{
		pt[0].x = x1; pt[0].y = y1;
		pt[1].x = x2; pt[1].y = y2;

		SelectObject (hdc, CreatePen (PS_DASH, LineWidth, RGB (0, 0 ,0)));
		Polyline(hdc,pt,2);
		DeleteObject (SelectObject (hdc, GetStockObject(BLACK_PEN)));
	}
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CBoundsRechteckH(HDC hdc, int x1, int y1, int x2, int y2, int LineWidth, int LineType)
{  
	POINT  pt[2];

	if (LineType > 0)
	{
		pt[0].x = x1; pt[0].y = y1;
		pt[1].x = x2; pt[1].y = y2;

		SelectObject (hdc, CreatePen (PS_DASH, LineWidth, RGB (0, 0 ,0)));
		Polyline(hdc,pt,2);
		DeleteObject (SelectObject (hdc, GetStockObject(BLACK_PEN)));
	}
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CWinkel(HDC hdc, float winkel, float C, float Cmax, 
						 int Mx, int My, float l, 
						 int LineWidth, RGBColor Farbe)
{
	double x  = 0.;

	float  Px = (float) 0.;
	float  Py = (float) 0.;

	if (C > Cmax)
		x = l;
	else
		x = C*l/Cmax;

	Px = (float) Mx + (float) (x*sin((double) winkel*0.0174533));
	Py = (float) My - (float) (x*cos((double) winkel*0.0174533));

	CLinie(hdc,Mx,My,(int) Px,(int) Py,LineWidth,Farbe);
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CLinie(HDC hdc, int x1, int y1, int x2, int y2, int LineWidth, RGBColor Farbe)
{  
	POINT  pt[2];

	pt[0].x = x1; pt[0].y = y1;
	pt[1].x = x2; pt[1].y = y2;

	SelectObject (hdc, CreatePen (PS_SOLID, LineWidth, RGB (Farbe.red, Farbe.green, Farbe.blue)));
	Polyline(hdc,pt,2);
	DeleteObject (SelectObject (hdc, GetStockObject(BLACK_PEN)));
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CRechteck(HDC hdc, int x1, int y1, int x2, int y2, int LineWidth, RGBColor Farbe)
{      
	POINT  pt[5];

	SelectObject (hdc, CreatePen (PS_SOLID, LineWidth, RGB (Farbe.red, Farbe.green, Farbe.blue)));
	pt[0].x = x1; pt[0].y = y1;
	pt[1].x = x2; pt[1].y = y1;
	pt[2].x = x2; pt[2].y = y2;
	pt[3].x = x1; pt[3].y = y2;
	pt[4].x = x1; pt[4].y = y1;

	Polyline(hdc,pt,5);
	DeleteObject (SelectObject (hdc, GetStockObject(BLACK_PEN)));
}      

/**********************************************************************
***********************************************************************
**********************************************************************/

void CFillRechteck(HDC hdc, int x1, int y1, int x2, int y2, int LineWidth, RGBColor Farbe)
{      
	HBRUSH hBrush;
	POINT  pt[5];

	pt[0].x = x1; pt[0].y = y1;
	pt[1].x = x2; pt[1].y = y1;
	pt[2].x = x2; pt[2].y = y2;
	pt[3].x = x1; pt[3].y = y2;
	pt[4].x = x1; pt[4].y = y1;

	SelectObject (hdc, CreatePen (PS_SOLID, LineWidth, RGB (Farbe.red, Farbe.green, Farbe.blue)));
	hBrush = (HBRUSH) SelectObject (hdc, CreateSolidBrush (RGB (Farbe.red, Farbe.green, Farbe.blue)));
	Polygon(hdc,pt,5);
	DeleteObject (SelectObject (hdc, hBrush));
	DeleteObject (SelectObject (hdc, GetStockObject(BLACK_PEN)));
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CFillPatternRechteck(HDC hdc, int x1, int y1, int x2, int y2, int PatternID)
{      
	BITMAP		bm;
	HBITMAP		hbm;
	HDC				hMemDC;

	int 			bmWidth   = 8;
	int				bmHeight  = 8;
	int 			LineWidth = 1; 

	int				yu, y;
	int				xr, x;

	int				bmH, bmW;

	int				Test = 0;
		 
//*********************************************************************

	bmWidth					= BitPattern[PatternID].Width;
	bmHeight					= BitPattern[PatternID].Height;

	bm.bmType				= 0;
	bm.bmWidthBytes	= 2;
	bm.bmPlanes			= 1;
	bm.bmBitsPixel		= 1;
	bm.bmWidth				= BitPattern[PatternID].Width;
	bm.bmHeight			= BitPattern[PatternID].Height;
	bm.bmBits				= BitPattern[PatternID].bBitmap;

	hbm = CreateBitmapIndirect(&bm);

	hMemDC = CreateCompatibleDC(hdc);
	SelectObject(hMemDC,hbm);

	if (Test)
	{
		BitBlt(hdc,200,200,bm.bmWidth,bm.bmHeight,hMemDC,0,0,SRCCOPY);
	}
	else
	{
		bmH = bmHeight*LineWidth;
		bmW = bmWidth*LineWidth;

		yu  = y2 - bmH;
		y   = y1;

		xr  = x2 - bmW;

		while (y < yu)
		{
			x	 = x1;

			while (x < xr)
			{	 
				StretchBlt(hdc,x,y,bmW,bmH,hMemDC,0,0,bmWidth,bmHeight,SRCCOPY);
				x += bmW - 1;
			}

			y += bmH - 1;
		}

		bmH = yu - (y-bmH);
		yu  = bmH/LineWidth;

		if (bmH > 0)
		{
			x	 = x1;

			while (x < xr)
			{	 
				StretchBlt(hdc,x,y,bmW,bmH,hMemDC,0,0,bmWidth,yu,SRCCOPY);
				x += bmW - 1;
			}
		}
	}

	DeleteDC (hMemDC);
	DeleteObject (hbm);

	CRechteck(hdc, x1, y1, x2, y2, LineWidth, BLACK);
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CCircle_outlined(HDC hdc, int x, int y, int Radius, int LineWidth, RGBColor Farbe)
{      
 	SelectObject (hdc, CreatePen (PS_SOLID, LineWidth, RGB (Farbe.red, Farbe.green, Farbe.blue)));
  Ellipse(hdc, x -  Radius, y -  Radius, x +  Radius, y +  Radius);
  DeleteObject (SelectObject (hdc, GetStockObject(BLACK_PEN)));
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CCircle_filled(HDC hdc, int x, int y, int Radius, int LineWidth, RGBColor Farbe)
{      
  HBRUSH hBrush;

  hBrush = (HBRUSH) SelectObject (hdc, CreateSolidBrush (RGB (Farbe.red, Farbe.green, Farbe.blue)));
	CCircle_outlined(hdc,x,y,Radius,LineWidth,Farbe);
  DeleteObject (SelectObject (hdc, hBrush));
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CRhombus_outlined(HDC hdc, int px, int py, int Abstand, int LineWidth, RGBColor Farbe)
{
	POINT  pt[5];

  SelectObject (hdc, CreatePen (PS_SOLID, LineWidth, RGB (Farbe.red, Farbe.green, Farbe.blue)));
  pt[0].x = px-Abstand; pt[0].y = py;
  pt[1].x = px       ; pt[1].y = py-Abstand;
	pt[2].x = px+Abstand; pt[2].y = py;
	pt[3].x = px       ; pt[3].y = py+Abstand;
  pt[4].x = px-Abstand; pt[4].y = py;

  Polygon(hdc,pt,5);
  DeleteObject (SelectObject (hdc, GetStockObject(BLACK_PEN)));
}
      
/**********************************************************************
***********************************************************************
**********************************************************************/

void CRhombus_filled(HDC hdc, int px, int py, int Abstand, int LineWidth, RGBColor Farbe)
{      
  HBRUSH hBrush;

  hBrush = (HBRUSH) SelectObject (hdc, CreateSolidBrush (RGB (Farbe.red, Farbe.green, Farbe.blue)));
	CRhombus_outlined(hdc,px,py,Abstand,LineWidth,Farbe);
  DeleteObject (SelectObject (hdc, hBrush));
}
      
/**********************************************************************
***********************************************************************
**********************************************************************/

void CTriangle_outlined(HDC hdc, int px, int py, int Abstand, int LineWidth, RGBColor Farbe)
{
	POINT  pt[4];

  SelectObject (hdc, CreatePen (PS_SOLID, LineWidth, RGB (Farbe.red, Farbe.green, Farbe.blue)));
  pt[0].x = px-Abstand; pt[0].y = py+Abstand;
  pt[1].x = px       ; pt[1].y = py-Abstand;
	pt[2].x = px+Abstand; pt[2].y = py+Abstand;
  pt[3].x = px-Abstand; pt[3].y = py+Abstand;

  Polygon(hdc,pt,4);
  DeleteObject (SelectObject (hdc, GetStockObject(BLACK_PEN)));
}
      
/**********************************************************************
***********************************************************************
**********************************************************************/

void CTriangle_filled(HDC hdc, int px, int py, int Abstand, int LineWidth, RGBColor Farbe)
{
  HBRUSH hBrush;

  hBrush = (HBRUSH) SelectObject (hdc, CreateSolidBrush (RGB (Farbe.red, Farbe.green, Farbe.blue)));
  CTriangle_outlined(hdc, px, py, Abstand, LineWidth, Farbe);
  DeleteObject (SelectObject (hdc, hBrush));
}
/**********************************************************************
***********************************************************************
**********************************************************************/

void CSquare_outlined(HDC hdc, int px, int py, int Abstand, int LineWidth, RGBColor Farbe)
{      
  CRechteck(hdc,px-Abstand+1,py-Abstand+1,px+Abstand-1,py+Abstand-1,LineWidth,Farbe);
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CSquare_filled(HDC hdc, int px, int py, int Abstand, int LineWidth, RGBColor Farbe)
{      
  CFillRechteck(hdc,px-Abstand+1,py-Abstand+1,px+Abstand-1,py+Abstand-1,LineWidth,Farbe);
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CCross(HDC hdc, int px, int py, int Abstand, int LineWidth, RGBColor Farbe)
{      
	POINT  	pt[6];
	int    	not_used;
	RGBColor not_used_Color;
	
	not_used 				= LineWidth;
	not_used_Color	= Farbe;
	
  pt[0].x = px-Abstand; pt[0].y = py;
  pt[1].x = px       ; pt[1].y = py;
	pt[2].x = px       ; pt[2].y = py+Abstand;
	pt[3].x = px       ; pt[3].y = py-Abstand;
  pt[4].x = px       ; pt[4].y = py;
  pt[5].x = px+Abstand; pt[5].y = py;

  Polygon(hdc,pt,6);
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CCross_diag(HDC hdc, int px, int py, int Abstand, int LineWidth, RGBColor Farbe)
{      
	CLinie(hdc,px-Abstand,py-Abstand,px+Abstand,py+Abstand,LineWidth,Farbe);
	CLinie(hdc,px+Abstand,py-Abstand,px-Abstand,py+Abstand,LineWidth,Farbe);
}
      
/**********************************************************************
***********************************************************************
**********************************************************************/

void CPlotPolyline(HDC hdc, POINT *pt, unsigned long iCount, int LineWidth, RGBColor Farbe)
{
	if (iCount < maxPTData)
	{
 		SelectObject (hdc, CreatePen (PS_SOLID, LineWidth, RGB (Farbe.red, Farbe.green, Farbe.blue)));
		Polyline (hdc,pt,(int) (iCount+1));
		DeleteObject (SelectObject (hdc, GetStockObject(BLACK_PEN)));
  }
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CPlotPolygon(HDC hdc, POINT *pt, unsigned long iCount, int LineWidth, RGBColor Farbe)
{
	if (iCount < maxPTData)
	{
 		SelectObject (hdc, CreatePen (PS_SOLID, LineWidth, RGB (Farbe.red, Farbe.green, Farbe.blue)));
		Polygon (hdc,pt,(int) (iCount+1));
		DeleteObject (SelectObject (hdc, GetStockObject(BLACK_PEN)));
  }   
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void CPlotFillPoly(HDC hdc, POINT *pt, unsigned long iCount, int LineWidth, RGBColor Farbe)
{
  HBRUSH hBrush;

	if (iCount < maxPTData)
	{
 		SelectObject (hdc, CreatePen (PS_SOLID, LineWidth, RGB (Farbe.red, Farbe.green, Farbe.blue)));
		hBrush = (HBRUSH) SelectObject (hdc, CreateSolidBrush (RGB (Farbe.red, Farbe.green, Farbe.blue)));
		Polygon(hdc,pt,(int) (iCount+1));
		DeleteObject (SelectObject (hdc, hBrush));
		DeleteObject (SelectObject (hdc, GetStockObject(BLACK_PEN)));
  }
}

/**********************************************************************
***********************************************************************
**********************************************************************/


void Linie(HDC hdc, int x1, int y1, int x2, int y2, int LineWidth)
{      
	POINT  pt[2];

	LineWidth = 0;

	pt[0].x = x1; pt[0].y = y1;
	pt[1].x = x2; pt[1].y = y2;

	Polyline(hdc,pt,2);
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void Rechteck(HDC hdc, int x1, int y1, int x2, int y2, int LineWidth)
{      
	POINT  pt[5];

	LineWidth = 0;

	pt[0].x = x1; pt[0].y = y1;
	pt[1].x = x2; pt[1].y = y1;
	pt[2].x = x2; pt[2].y = y2;
	pt[3].x = x1; pt[3].y = y2;
	pt[4].x = x1; pt[4].y = y1;

	Polyline(hdc,pt,5);
}      

/**********************************************************************
***********************************************************************
**********************************************************************/

void FillRechteck(HDC hdc, int x1, int y1, int x2, int y2, int LineWidth)
{      
	POINT  pt[5];

	pt[0].x = x1; pt[0].y = y1;
	pt[1].x = x2; pt[1].y = y1;
	pt[2].x = x2; pt[2].y = y2;
	pt[3].x = x1; pt[3].y = y2;
	pt[4].x = x1; pt[4].y = y1;

	PlotFillPoly(hdc,pt,4,LineWidth);
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void Circle_outlined(HDC hdc, int x, int y, int Radius, int LineWidth)
{      
	LineWidth = 0;
	
  Ellipse(hdc, x -  Radius, y -  Radius, x +  Radius, y +  Radius);
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void Circle_filled(HDC hdc, int x, int y, int Radius, int LineWidth)
{      
	HBRUSH hBrush;

	hBrush = (HBRUSH) SelectObject (hdc, CreateSolidBrush (RGB (0,0,0)));
	Circle_outlined(hdc,x,y,Radius,LineWidth);
	DeleteObject (SelectObject (hdc, hBrush));
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void Rhombus_outlined(HDC hdc, int px, int py, int Abstand, int LineWidth)
{
	POINT  pt[5];

	LineWidth = 0;

  pt[0].x = px-Abstand; pt[0].y = py;
  pt[1].x = px       ; pt[1].y = py-Abstand;
	pt[2].x = px+Abstand; pt[2].y = py;
	pt[3].x = px       ; pt[3].y = py+Abstand;
  pt[4].x = px-Abstand; pt[4].y = py;

  Polygon(hdc,pt,5);
}
      
/**********************************************************************
***********************************************************************
**********************************************************************/

void Rhombus_filled(HDC hdc, int px, int py, int Abstand, int LineWidth)
{      
  HBRUSH hBrush;

  hBrush = (HBRUSH) SelectObject (hdc, CreateSolidBrush (RGB (0,0,0)));
	Rhombus_outlined(hdc,px,py,Abstand,LineWidth);
  DeleteObject (SelectObject (hdc, hBrush));
}
      
/**********************************************************************
***********************************************************************
**********************************************************************/

void Square_outlined(HDC hdc, int px, int py, int Abstand, int LineWidth)
{      
  Rechteck(hdc,px-Abstand+1,py-Abstand+1,px+Abstand-1,py+Abstand-1,LineWidth);
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void Square_filled(HDC hdc, int px, int py, int Abstand, int LineWidth)
{      
  FillRechteck(hdc,px-Abstand+1,py-Abstand+1,px+Abstand-1,py+Abstand-1,LineWidth);
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void Cross(HDC hdc, int px, int py, int Abstand, int LineWidth)
{      
	POINT  pt[6];

	LineWidth = 0;

	pt[0].x = px-Abstand; pt[0].y = py;
	pt[1].x = px       ; pt[1].y = py;
	pt[2].x = px       ; pt[2].y = py+Abstand;
	pt[3].x = px       ; pt[3].y = py-Abstand;
	pt[4].x = px       ; pt[4].y = py;
	pt[5].x = px+Abstand; pt[5].y = py;

	Polygon(hdc,pt,6);
}
      
/**********************************************************************
***********************************************************************
**********************************************************************/

void Cross_diag(HDC hdc, int px, int py, int Abstand, int LineWidth)
{      
	Linie(hdc,px-Abstand,py-Abstand,px+Abstand,py+Abstand,LineWidth);
	Linie(hdc,px+Abstand,py-Abstand,px-Abstand,py+Abstand,LineWidth);
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void PlotPolyline(HDC hdc, POINT *pt, unsigned long iCount, int LineWidth)
{
  LineWidth = 0;

	if (iCount < maxPTData)
    Polyline(hdc,pt,(int) (iCount+1));
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void PlotPolygon(HDC hdc, POINT *pt, unsigned long iCount, int LineWidth)
{
  LineWidth = 0;
	if (iCount < maxPTData)
    Polygon(hdc,pt,(int) (iCount+1));
}

/**********************************************************************
***********************************************************************
**********************************************************************/

void PlotFillPoly(HDC hdc, POINT *pt, unsigned long iCount, int LineWidth)
{
  HBRUSH hBrush;

  LineWidth = 0;
  
	if (iCount < maxPTData)
	{
		hBrush = (HBRUSH) SelectObject (hdc, CreateSolidBrush (RGB (0,0,0)));
		Polygon(hdc,pt,(int) (iCount+1));		 
		DeleteObject (SelectObject (hdc, hBrush));
  }
}


