/*********************************************************************
*                                                                    *
*     Window.c                                                       *
*                                                                    *
*     Programmautor: Rainer Sieger, Universitt Bremen               *
*     Datei erstellt am: 11.10.96                                    *
*     Letzte nderung: 03.01.2001                                    *
*     Windows only                                                   *
*                                                                    *
*********************************************************************/

#include <windows.h>
#include <stdio.h>

#include "Struct.h"
#include "Globals.h"

#include "SetMinMax.h"
#include "FindItem.h"
#include "Win_GUI.h"
#include "Window.h"
#include "Dialog.h"
#include "Do_GraphicDialog.h"
#include "Do_TernaryModeDialog.h"
#include "Do_PhaseModeDialog.h"
#include "Do_XAxisDialog.h"
#include "Do_YAxisDialog.h"

/*********************************************************************/
/*********************************************************************/
/*********************************************************************/

void GetWindowSize(HWND hwnd, HWND hStatusWnd, int *WindowWidth, int *WindowHeight) 
{
	RECT	clrect, strect;

	GetClientRect(hwnd, &clrect);
	GetClientRect(hStatusWnd, &strect);

  *WindowHeight = clrect.bottom-strect.bottom;
  *WindowWidth  = clrect.right;
}

/*********************************************************************/
/*********************************************************************/
/*********************************************************************/

void GetFrameWindowPos(HWND hwndFrame, int *left, int *top, int *WindowWidth, int *WindowHeight) 
{
	RECT	wndrect;
		
  GetWindowRect(hwndFrame, &wndrect);
	
	*left = wndrect.left;
	*top	= wndrect.top;
	*WindowWidth  = wndrect.right - wndrect.left;
	*WindowHeight = wndrect.bottom - wndrect.top;
}

/*********************************************************************/
/*********************************************************************/
/*********************************************************************/

void GetClientWindowPos(HWND hwndFrame, HWND hwnd, int *left, int *top, int *WindowWidth, int *WindowHeight) 
{
	RECT	clrect, wndrect;
	
	int		leftClient, topClient;

	int		Border       = 0;
	int		HeaderHeight = 0;
	
  GetWindowRect(hwndFrame, &wndrect);
	GetClientRect(hwndFrame, &clrect);
	
	Border 			 = ((wndrect.right - wndrect.left) - (clrect.right - clrect.left))/2;
	HeaderHeight = (wndrect.bottom - wndrect.top) - (clrect.bottom - clrect.top) - Border;
	
	leftClient   = wndrect.left + Border;
	topClient		 = wndrect.top + HeaderHeight; 

  GetWindowRect(hwnd, &wndrect);
	GetClientRect(hwnd, &clrect);
	
	*left = wndrect.left - leftClient;
	*top	= wndrect.top - topClient;
	*WindowWidth  = wndrect.right - wndrect.left;
	*WindowHeight = wndrect.bottom - wndrect.top;
}

/*********************************************************************/
/*********************************************************************/
/*********************************************************************/

void ResizeWindow(HWND hwndFrame, HWND hwnd, HWND hStatusWnd, int WindowLeft, int WindowTop, int WindowWidth, int WindowHeight)
{
	RECT	clrect, strect, wndrect;

  int  xBorder            = 0;
  int  yBorder            = 0;

	int  maxWindowWidth			= 0;
	int  maxWindowHeight		= 0;
	int	 NewWindowWidth			= 0;
	int	 NewWindowHeight		= 0;
	
  GetWindowRect(hwnd, &wndrect);
  GetClientRect(hwnd, &clrect);

	GetClientRect(hStatusWnd, &strect);

	xBorder = ((wndrect.right - wndrect.left) - (clrect.right - clrect.left))/2;
  yBorder = (wndrect.bottom - wndrect.top) - (clrect.bottom - strect.bottom);

	GetClientRect(hwndFrame, &clrect);
	maxWindowWidth  = clrect.right;
	maxWindowHeight = clrect.bottom;

  NewWindowWidth  = WindowWidth+2*xBorder;
	NewWindowHeight = WindowHeight+yBorder;
	
	if ((WindowWidth<maxWindowWidth) && (WindowHeight<maxWindowHeight))
	{
		MoveWindow(hwnd,WindowLeft,WindowTop,NewWindowWidth,NewWindowHeight,TRUE);
	}
	else
	{
		if ((WindowWidth>=maxWindowWidth) && (WindowHeight>=maxWindowHeight))
		{
			MoveWindow(hwnd,0,0,maxWindowWidth,maxWindowHeight,TRUE);
		}
		else
		{
			if (WindowWidth>=maxWindowWidth)
				MoveWindow(hwnd,0,WindowTop,maxWindowWidth,NewWindowHeight,TRUE);
		
			if (WindowHeight>=maxWindowHeight)
				MoveWindow(hwnd,WindowLeft,0,NewWindowWidth,maxWindowHeight,TRUE);
		}
	}
	return;
}

/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/

void Do_ContentClick (HWND hwndFrame, HWND hwnd, HWND hStatusWnd, HINSTANCE hInst, int Mx, int My)
{
	switch (Mode)
	{
		case VerticalMode:
		case HorizontalMode:
		  ItemNo = FindItem(Mx,My);

			switch (ItemNo)
			{
  			case -1: 
					if (DialogBox (hInst, "GRAPHICDIALOG", hwnd, (DLGPROC) Do_GraphicDialog))
					{
						GetClientWindowPos(hwndFrame,hwnd,&gClientDataPlotWindowX,&gClientDataPlotWindowY,&gClientDataPlotWindowWidth,&gClientDataPlotWindowHeight);
						ResizeWindow (hwndFrame, hwnd, hStatusWnd, gClientDataPlotWindowX, gClientDataPlotWindowY, gDataPlotWindowWidth, gDataPlotWindowHeight);
						InvalidateRgn (hwnd, NULL, TRUE); 
					}
					break;

				case -2: 
					DialogBox (hInst, "XAXISDIALOG", hwnd, (DLGPROC) Do_XAxisDialog);
					InvalidateRgn (hwnd, NULL, TRUE); 
  				break;

  			default:
					Item[0].what = ItemNo;

					switch (Mode)
					{
						case VerticalMode:
							if (My < C[ItemNo].P[2])
								DialogBox (hInst, "PARAMETERAXISDIALOG", hwnd, (DLGPROC) Do_ParameterAxisDialog);
							else
								DialogBox (hInst, "PARAMETEROPTIONDIALOG", hwnd, (DLGPROC) Do_ParameterOptionDialog);
							break;

						case HorizontalMode:
							if (Mx < C[ItemNo].P[1])
								DialogBox (hInst, "PARAMETERAXISDIALOG", hwnd, (DLGPROC) Do_ParameterAxisDialog);
							else
								DialogBox (hInst, "PARAMETEROPTIONDIALOG", hwnd, (DLGPROC) Do_ParameterOptionDialog);
							break;

						default:
							break;
					}

					InvalidateRgn (hwnd, NULL, TRUE); 
  				break;
 			}
			break;

		case TernaryMode:
		  if ((ItemNo = FindCheckBoxTernaryMode(Mx,My)) != 0)
			{
				if (ItemNo > 0)
				{
					if (ST[ItemNo].GPlotData > 0)
						ST[ItemNo].GPlotData = 0;
					else
						ST[ItemNo].GPlotData = 1;

					InvalidateRgn (hwnd, NULL, TRUE); 
				}
				else
				{
					ItemNo *= -1;
					DialogBox (hInst, "TERNARYMODEDIALOG", hwnd, (DLGPROC) Do_TernaryModeDialog);
					InvalidateRgn (hwnd, NULL, TRUE); 
				}
			}
			else
			{
				if (DialogBox (hInst, "TERNARYMODEGRAPHICDIALOG", hwnd, (DLGPROC) Do_GraphicDialog))
				{
					GetClientWindowPos(hwndFrame,hwnd,&gClientDataPlotWindowX,&gClientDataPlotWindowY,&gClientDataPlotWindowWidth,&gClientDataPlotWindowHeight);
					ResizeWindow (hwndFrame, hwnd, hStatusWnd, gClientDataPlotWindowX, gClientDataPlotWindowY, gDataPlotWindowWidth, gDataPlotWindowHeight);
					InvalidateRgn (hwnd, NULL, TRUE); 
				}
			}
			break;

		case PhaseMode:
		  if ((ItemNo = FindCheckBoxPhaseMode(Mx,My)) != 0)
			{
				if (ItemNo > 0)
				{
					if (ST[ItemNo].GPlotData > 0)
						ST[ItemNo].GPlotData = 0;
					else
						ST[ItemNo].GPlotData = 1;

					InvalidateRgn (hwnd, NULL, TRUE); 
				}
				else
				{
					ItemNo *= -1;
					DialogBox (hInst, "PHASEMODEDIALOG", hwnd, (DLGPROC) Do_PhaseModeDialog);
					InvalidateRgn (hwnd, NULL, TRUE); 
				}
			}
			else
			{
				if (DialogBox (hInst, "TERNARYMODEGRAPHICDIALOG", hwnd, (DLGPROC) Do_GraphicDialog))
				{
					GetClientWindowPos(hwndFrame,hwnd,&gClientDataPlotWindowX,&gClientDataPlotWindowY,&gClientDataPlotWindowWidth,&gClientDataPlotWindowHeight);
					ResizeWindow (hwndFrame, hwnd, hStatusWnd, gClientDataPlotWindowX, gClientDataPlotWindowY, gDataPlotWindowWidth, gDataPlotWindowHeight);
					InvalidateRgn (hwnd, NULL, TRUE); 
				}
			}
			break;

		default:
			break;
	}

}

/*********************************************************************/

void EraseWindow(int theWindow)
{
	theWindow = 0;
  return;
}

/*********************************************************************/
/*********************************************************************/
/*********************************************************************/

void maxWindowSize(int *top,int *left,int *height,int *width)
{
  *top    = 50;
  *left   = 50;
  *width  = GetSystemMetrics(SM_CXSCREEN);
  *height = GetSystemMetrics(SM_CYSCREEN);
}

